@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.25 2007/01/12 00:08:15 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.26 2007/12/29 21:17:05 edi Exp $
 
 ;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
 
@@ -114,7 +114,7 @@ IN-MEMORY streams."))
   (:report (lambda (condition stream)
              (format stream "~S is closed."
                      (stream-error-stream condition))))
-  (:documentation "An error that is signaled when someone is trying
+  (:documentation "An error that is signalled when someone is trying
 to read from or write to a closed IN-MEMORY stream."))
 
 #+:cmu
