@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/input.lisp,v 1.48 2007/09/06 23:19:24 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/input.lisp,v 1.51 2007/12/29 22:58:43 edi Exp $
 
 ;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
 
@@ -140,10 +140,15 @@ slot."
       (setq last-octet octet)
       (or octet :eof))))
 
+(defgeneric unread-char% (char-code flexi-input-stream)
+  (:documentation "Used internally to put a character denoted by the
+character code CHAR-CODE which was already read back on the stream.
+Uses the OCTET-STACK slot and decrements the POSITION slot
+accordingly."))
+
 (defmethod unread-char% (char-code (flexi-input-stream flexi-input-stream))
-  "Used internally to put a character denoted by the character code
-CHAR-CODE which was already read back on the stream.  Uses the
-OCTET-STACK slot and decrements the POSITION slot accordingly."
+  "The default method which is un-optimized and uses TRANSLATE-CHAR to
+figure out which octets to put on the octet stack."
   (declare (optimize speed) (inline translate-char))
   (with-accessors ((position flexi-stream-position)
                    (octet-stack flexi-stream-octet-stack)
@@ -154,6 +159,89 @@ OCTET-STACK slot and decrements the POSITION slot accordingly."
       (decf position (length octets-read))
       (setq octet-stack (append octets-read octet-stack)))))
 
+(defmethod unread-char% (char-code (flexi-input-stream flexi-latin-1-input-stream))
+  "For ISO-8859-1 we can simply put the character code itself on the
+octet stack."
+  (declare (optimize speed))
+  (with-accessors ((position flexi-stream-position)
+                   (octet-stack flexi-stream-octet-stack))
+      flexi-input-stream
+    (declare (integer position))
+    (decf position)
+    (push char-code octet-stack)))
+
+(defmethod unread-char% (char-code (flexi-input-stream flexi-ascii-input-stream))
+  "For ASCII we can simply put the character code itself on the octet
+stack."
+  (declare (optimize speed))
+  (with-accessors ((position flexi-stream-position)
+                   (octet-stack flexi-stream-octet-stack))
+      flexi-input-stream
+    (declare (integer position))
+    (decf position)
+    (push char-code octet-stack)))
+
+(defmethod unread-char% (char-code (flexi-input-stream flexi-8-bit-input-stream))
+  "For 8-bit encodings we just have to put one octet on the octet
+stack which we can look up in the encoding hash."
+  (declare (optimize speed))
+  (with-accessors ((position flexi-stream-position)
+                   (octet-stack flexi-stream-octet-stack)
+                   (encoding-hash flexi-stream-encoding-hash))
+      flexi-input-stream
+    (declare (integer position))
+    (decf position)
+    (push (gethash char-code encoding-hash) octet-stack)))
+
+(defmethod unread-char% ((char-code (eql #.(char-code #\Newline)))
+                         (flexi-input-stream flexi-cr-8-bit-input-stream))
+  "A kind of `safety net' for the optimized 8-bit versions of
+UNREAD-CHAR% which checks for the single case where more than one
+octet has to be put on the octet stack."
+  (declare (optimize speed))
+  (with-accessors ((position flexi-stream-position)
+                   (octet-stack flexi-stream-octet-stack)
+                   (external-format flexi-stream-external-format))
+      flexi-input-stream
+    (declare (integer position))
+    ;; note that below we use the knowledge that in all 8-bit encodings
+    ;; #\Return and #\Linefeed are mapped to the same octets
+    (case (external-format-eol-style external-format)
+      (:crlf
+       (decf position 2)
+       (push #.(char-code #\Linefeed) octet-stack)
+       (push #.(char-code #\Return) octet-stack))
+      (otherwise
+       (decf position)
+       (push #.(char-code #\Return) octet-stack)))))
+
+#+:lispworks
+(defmethod unread-char% ((char-code (eql #.(char-code #\Newline)))                         
+                         (flexi-input-stream flexi-binary-cr-8-bit-input-stream))
+  "A kind of `safety net' for the optimized 8-bit versions of
+UNREAD-CHAR% which checks for the single case where more than one
+octet has to be put on the octet stack.
+
+This method \(identical to the one defined directly above) exists only
+for LispWorks' \"binary\" streams and must be there due to the
+slightly clunky class hierarchy."
+  (declare (optimize speed))
+  (with-accessors ((position flexi-stream-position)
+                   (octet-stack flexi-stream-octet-stack)
+                   (external-format flexi-stream-external-format))
+      flexi-input-stream
+    (declare (integer position))
+    ;; note that below we use the knowledge that in all 8-bit encodings
+    ;; #\Return and #\Linefeed are mapped to the same octets
+    (case (external-format-eol-style external-format)
+      (:crlf
+       (decf position 2)
+       (push #.(char-code #\Linefeed) octet-stack)
+       (push #.(char-code #\Return) octet-stack))
+      (otherwise
+       (decf position)
+       (push #.(char-code #\Return) octet-stack)))))
+
 (defmacro define-char-reader ((stream-var stream-class) &body body)
   "Helper macro to define methods for STREAM-READ-CHAR.  Defines a
 method for the class STREAM-CLASS using the variable STREAM-VAR and
@@ -450,8 +538,8 @@ being used by the stream."
              :format-control "No character to unread from this stream \(or external format has changed or last reading operation was binary)."))
     (unless (= (char-code char) last-char-code)
       (error 'flexi-stream-simple-error
-             :format-control "Last character read was different from ~S."
-             :format-arguments (list char)))
+             :format-control "Last character read (~S) was different from ~S."
+             :format-arguments (list (code-char last-char-code) char)))
     (unread-char% last-char-code stream)
     (setq last-char-code nil)
     nil))
