@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/output.lisp,v 1.42 2007/09/13 19:35:49 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/output.lisp,v 1.44 2007/12/29 22:23:23 edi Exp $
 
 ;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
 
@@ -283,6 +283,12 @@ an internal buffer and uses repeated calls to WRITE-SEQUENCE to write
 to the underlying stream."
   (declare (optimize speed)
            (type (integer 0 *) start end))
+  ;; don't use this optimized method for bivalent character streams on
+  ;; LispWorks, as it currently gets confused by the fill pointer
+  #+:lispworks
+  (unless (typep stream 'flexi-binary-output-stream)
+    (return-from stream-write-sequence
+      (call-next-method)))
   (let* ((buffer (make-array (+ +buffer-size+ 20)
                              :element-type '(unsigned-byte 8)
                              :fill-pointer 0))
