@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-USER; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/packages.lisp,v 1.28 2007/09/06 23:19:24 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/packages.lisp,v 1.30 2007/10/11 20:23:53 edi Exp $
 
 ;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
 
@@ -29,7 +29,7 @@
 
 (in-package :cl-user)
 
-(unless (find-symbol "STREAM-FILE-POSITION" :trivial-gray-streams)
+(unless (find-symbol (symbol-name :stream-file-position) :trivial-gray-streams)
   (error "You need a newer version of TRIVIAL-GRAY-STREAMS."))
 
 (defpackage :flexi-streams
