@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/stream.lisp,v 1.50 2007/09/06 23:46:29 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/stream.lisp,v 1.53 2007/12/29 22:26:04 edi Exp $
 
 ;;; Copyright (c) 2005-2007, Dr. Edmund Weitz.  All rights reserved.
 
@@ -65,18 +65,18 @@ capabilities."))
   (:report (lambda (condition stream)
              (format stream "Element type ~S not allowed."
                      (flexi-stream-element-type-error-element-type condition))))
-  (:documentation "Errors of this type are signaled if the flexi
+  (:documentation "Errors of this type are signalled if the flexi
 stream has a wrong element type."))
 
 (define-condition flexi-stream-encoding-error (flexi-stream-simple-error)
   ()
-  (:documentation "Errors of this type are signaled if there is an
+  (:documentation "Errors of this type are signalled if there is an
 encoding problem."))
 
 (define-condition flexi-stream-position-spec-error (flexi-stream-simple-error)
   ((position-spec :initarg :position-spec
                   :reader flexi-stream-position-spec-error-position-spec))
-  (:documentation "Errors of this type are signaled if an
+  (:documentation "Errors of this type are signalled if an
 erroneous position spec is used in conjunction with
 FILE-POSITION."))
 
@@ -232,11 +232,16 @@ MAKE-FLEXI-STREAM instead."))
 end-of-line conversion, i.e. for those where the end-of-line
 designator is /not/ the single character #\Linefeed."))
 
-(defclass flexi-8-bit-input-stream (flexi-input-stream)
+(defclass flexi-8-bit-stream (flexi-stream)
+  ((encoding-hash :accessor flexi-stream-encoding-hash))
+  (:documentation "The class for all flexi streams which use an 8-bit
+encoding and thus need an additional slot for the encoding hash."))
+
+(defclass flexi-8-bit-input-stream (flexi-input-stream flexi-8-bit-stream)
   ((encoding-table :accessor flexi-stream-encoding-table))
-  (:documentation "The class for all flexi input streams which
-use an 8-bit encoding and thus need an additional slot for the
-encoding table."))
+  (:documentation "The class for all flexi input streams which use an
+8-bit encoding and thus need an additional slot for the encoding
+table."))
 
 (defclass flexi-cr-8-bit-input-stream (flexi-cr-mixin flexi-8-bit-input-stream)
   ()
@@ -317,11 +322,10 @@ use the UTF-8 encoding."))
   (:documentation "Special class for flexi input streams which
 use the UTF-8 encoding /and/ need end-of-line conversion."))
 
-(defclass flexi-8-bit-output-stream (flexi-output-stream)
-  ((encoding-hash :accessor flexi-stream-encoding-hash))
-  (:documentation "The class for all flexi output streams which
-use an 8-bit encoding and thus need an additional slot for the
-encoding table."))
+(defclass flexi-8-bit-output-stream (flexi-output-stream flexi-8-bit-stream)
+  ()
+  (:documentation "The class for all flexi output streams which use an
+8-bit encoding."))
 
 (defclass flexi-cr-8-bit-output-stream (flexi-cr-mixin flexi-8-bit-output-stream)
   ()
@@ -404,9 +408,8 @@ use the UTF-8 encoding /and/ need end-of-line conversion."))
 
 (defclass flexi-8-bit-io-stream (flexi-8-bit-input-stream flexi-8-bit-output-stream flexi-io-stream)
   ()
-  (:documentation "The class for all flexi I/O streams which use
-an 8-bit encoding and thus need an additional slot for the
-encoding table."))
+  (:documentation "The class for all flexi I/O streams which use an
+8-bit encoding."))
   
 (defclass flexi-cr-8-bit-io-stream (flexi-cr-mixin flexi-8-bit-io-stream)
   ()
@@ -633,18 +636,34 @@ of its 8-bit encoding slots."
   "After we've changed the actual class of a flexi stream we may
 have to set the contents of the 8-bit enconding slots as well."
   (declare (optimize speed))
-  (set-encoding-table stream)
-  (set-encoding-hash stream))
+  (set-encoding-hash stream)
+  (set-encoding-table stream))
+
+(defgeneric set-encoding-hash (stream)
+  (:method (stream))
+  (:documentation "Sets the value of the ENCODING-HASH slot of
+STREAM if necessary.  The default method does nothing."))
 
 (defgeneric set-encoding-table (stream)
   (:method (stream))
   (:documentation "Sets the value of the ENCODING-TABLE slot of
 STREAM if necessary.  The default method does nothing."))
 
-(defgeneric set-encoding-hash (stream)
-  (:method (stream))
-  (:documentation "Sets the value of the ENCODING-HASH slot of
-STREAM if necessary.  The default method does nothing."))
+(defmethod set-encoding-hash ((stream flexi-8-bit-stream))
+  "Sets the value of the ENCODING-HASH slot of STREAM depending
+on its external format."
+  (declare (optimize speed))
+  (with-accessors ((external-format flexi-stream-external-format)
+                   (encoding-hash flexi-stream-encoding-hash))
+      stream
+    (let ((external-format-name (external-format-name external-format)))
+      (setq encoding-hash
+            (cond ((ascii-name-p external-format-name) +ascii-hash+)
+                  ((koi8-r-name-p external-format-name) +koi8-r-hash+)
+                  ((iso-8859-name-p external-format-name)
+                   (cdr (assoc external-format-name +iso-8859-hashes+ :test #'eq)))
+                  ((code-page-name-p external-format-name)
+                   (cdr (assoc (external-format-id external-format) +code-page-hashes+))))))))
 
 (defmethod set-encoding-table ((stream flexi-8-bit-input-stream))
   "Sets the value of the ENCODING-TABLE slot of STREAM depending
@@ -662,23 +681,6 @@ on its external format."
                   ((code-page-name-p external-format-name)
                    (cdr (assoc (external-format-id external-format) +code-page-tables+))))))))
 
-(defmethod set-encoding-hash ((stream flexi-8-bit-output-stream))
-  "Sets the value of the ENCODING-HASH slot of STREAM depending
-on its external format."
-  (declare (optimize speed))
-  (with-accessors ((external-format flexi-stream-external-format)
-                   (encoding-hash flexi-stream-encoding-hash))
-      stream
-    (let ((external-format-name (external-format-name external-format)))
-      (setq encoding-hash
-            (cond ((ascii-name-p external-format-name) +ascii-hash+)
-                  ((koi8-r-name-p external-format-name) +koi8-r-hash+)
-                  ((iso-8859-name-p external-format-name)
-                   (cdr (assoc external-format-name +iso-8859-hashes+ :test #'eq)))
-                  ((code-page-name-p external-format-name)
-                   (cdr (assoc (external-format-id external-format) +code-page-hashes+))))))))
-
-
 #+:cmu
 (defmethod input-stream-p ((stream flexi-io-stream))
   "Explicitly states whether this is an input stream."
