@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS-TEST; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/test/test.lisp,v 1.12 2007/03/09 01:14:30 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/test/test.lisp,v 1.17 2007/12/29 22:58:44 edi Exp $
 
 ;;; Copyright (c) 2006-2007, Dr. Edmund Weitz.  All rights reserved.
 
@@ -31,15 +31,22 @@
 
 (defvar *this-file* (load-time-value
                      (or #.*compile-file-pathname* *load-pathname*))
-  "The pathname of the file (`test.lisp') where this variable was
+  "The pathname of the file \(`test.lisp') where this variable was
 defined.")
 
+#+:lispworks
+(defun get-env-variable-as-directory (name)
+  (lw:when-let (string (lw:environment-variable name))
+    (when (plusp (length string))
+      (cond ((find (char string (1- (length string))) "\\/" :test #'char=) string)
+            (t (lw:string-append string "/"))))))
+
 (defvar *tmp-dir*
   (load-time-value
-    (merge-pathnames "flexi-streams-test/"
+    (merge-pathnames "odd-streams-test/"
                      #+:allegro (system:temporary-directory)
-                     #+:lispworks (pathname (or (lw:environment-variable "TEMP")
-                                                (lw:environment-variable "TMP")
+                     #+:lispworks (pathname (or (get-env-variable-as-directory "TEMP")
+                                                (get-env-variable-as-directory "TMP")
                                                 #+:win32 "C:/"
                                                 #-:win32 "/tmp/"))
                      #-(or :allegro :lispworks) #p"/tmp/"))
@@ -140,11 +147,13 @@ opened with the :DIRECTION keyword argument DIRECTION-OUT."
 binary streams.  Only used to test LispWorks-specific behaviour."
   (with-open-file (in path-in
                       :external-format '(:latin-1 :eol-style :lf)
+                      :element-type 'base-char
                       :direction direction-in
                       :if-does-not-exist :error
                       :if-exists :overwrite)
     (with-open-file (out path-out
                          :external-format '(:latin-1 :eol-style :lf)
+                         :element-type 'base-char
                          :direction direction-out
                          :direction :output
                          :if-does-not-exist :create
@@ -181,22 +190,23 @@ the :DIRECTION keyword when opening the files."
                 (flex::normalize-external-format external-format-in) direction-in
                 (flex::normalize-external-format external-format-out) direction-out)
         #+:lispworks
-        (copy-file full-path-in external-format-in
-                   full-path-out external-format-out
-                   direction-out direction-in)
+        (copy-file-lw full-path-in external-format-in
+                      full-path-out external-format-out
+                      direction-out direction-in)
         #+:lispworks
         (cond ((file-equal full-path-out full-path-orig)
                (incf *test-success-counter*))
               (t (format *error-output* "    Test failed!!!~%")))
+        #+:lispworks
         (terpri *error-output*)))))
 
 (defmacro with-test ((test-description) &body body)
   "Defines a test.  Two utilities are available inside of the body of
 the maco: The function FAIL, and the macro CHECK.  FAIL, the lowest
-level utility, marks the test defined by WITH-TEST as faided.  CHECK
+level utility, marks the test defined by WITH-TEST as failed.  CHECK
 checks whether its argument is true, otherwise it calls FAIL. If
 during evaluation of the specified expression any condition is
-signaled, this is also considered a failure.
+signalled, this is also considered a failure.
 
 WITH-TEST prints reports while the tests run.  It also increments
 *TEST-SUCCESS-COUNT* if a test completes successfully."
@@ -223,7 +233,7 @@ WITH-TEST prints reports while the tests run.  It also increments
 
 (defmacro using-values ((&rest values) &body body)
   "Executes BODY and feeds an element from VALUES to the USE-VALUE
-restart each time a FLEXI-STREAM-ENCODING-ERROR is signaled.  Signals
+restart each time a FLEXI-STREAM-ENCODING-ERROR is signalled.  Signals
 an error when there are more or less FLEXI-STREAM-ENCODING-ERRORs than
 there are elements in VALUES."
   (flex::with-unique-names (value-stack condition-counter)
@@ -233,13 +243,13 @@ there are elements in VALUES."
                        #'(lambda (c)
                            (declare (ignore c)) 
                            (unless ,value-stack
-                             (error "Too many FLEXI-STREAM-ENCODING-ERRORs signaled, expected only ~A."
+                             (error "Too many FLEXI-STREAM-ENCODING-ERRORs signalled, expected only ~A."
                                     ,(length values)))
                            (incf ,condition-counter)
                            (use-value (pop ,value-stack)))))
          (prog1 (progn ,@body)
            (when ,value-stack
-             (error "~A FLEXI-STREAM-ENCODING-ERRORs signaled, but ~A were expected."
+             (error "~A FLEXI-STREAM-ENCODING-ERRORs signalled, but ~A were expected."
                     ,condition-counter ,(length values))))))))
 
 (defun read-flexi-line (sequence external-format)
@@ -249,8 +259,9 @@ the external format EXTERNAL-FORMAT."
     (setq in (make-flexi-stream in :external-format external-format))
     (read-line in)))
 
-(defun encoding-error-handling-test()
-  (with-test ("Handling of encoding errors")
+(defun encoding-error-handling-test ()
+  "Tests several possible encoding errors and how they are handled."
+  (with-test ("Handling of encoding errors.")
     (let ((*substitution-char* #\?))
       ;; :ASCII doesn't have characters with char codes > 127
       (check (string= "a??" (read-flexi-line `(,(char-code #\a) 128 200) :ascii)))
@@ -281,7 +292,7 @@ the external format EXTERNAL-FORMAT."
       (check (string= "E" (using-values (#\E) (read-flexi-line `(#x01) :utf-16be))))
       (check (string= "R" (using-values (#\R) (read-flexi-line `(#xd8 #x01) :utf-16be))))
       (check (string= "T" (using-values (#\T) (read-flexi-line `(#xd8 #x01 #xdb #xff) :utf-16be))))
-      ;; the only case when error is signaled for UTF-32 is at end of file
+      ;; the only case when error is signalled for UTF-32 is at end of file
       ;; in the middle of 4-byte sequence, both for big and little endian
       (check (string= "Y" (using-values (#\Y) (read-flexi-line `(#x01) :utf-32le))))
       (check (string= "Y" (using-values (#\Y) (read-flexi-line `(#x01 #x01) :utf-32le))))
@@ -294,6 +305,22 @@ the external format EXTERNAL-FORMAT."
       (check (string= "aY" (using-values (#\Y)
                              (read-flexi-line `(#x00 #x00 #x00 ,(char-code #\a) #x01) :utf-32be)))))))
 
+(defun unread-char-test ()
+  "Tests whether UNREAD-CHAR behaves as expected."
+  (with-test ("UNREAD-CHAR behaviour.")
+    (flet ((test-one-file (file-name external-format)
+             (with-open-file (in (merge-pathnames file-name *this-file*)
+                                 :element-type 'flex:octet)
+               (setq in (make-flexi-stream in :external-format external-format))
+               (loop repeat 300
+                     for char = (read-char in)
+                     do (unread-char char in)
+                     (check (char= (read-char in) char))))))
+      (loop for (file-name symbols) in *test-files*
+            do (loop for symbol in symbols
+                     do (loop for (file-name . external-format) in (create-file-variants file-name symbol)
+                              do (test-one-file file-name external-format)))))))
+
 (defun run-tests ()
   "Applies COMPARE-FILES to all test scenarios created with
 CREATE-TEST-COMBINATIONS, runs test for handling of encoding errors,
@@ -308,6 +335,8 @@ and shows simple statistics at the end."
       (apply #'compare-files args))
     (incf no-tests)
     (encoding-error-handling-test)
+    (incf no-tests)
+    (unread-char-test)
     (format *error-output* "~%~%~:[~A of ~A tests failed..~;~*All ~A tests passed~].~%"
             (= no-tests *test-success-counter*) (- no-tests *test-success-counter*) no-tests)))
             
